@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // 'Group sockets'
 // Implementation
 
@@ -269,11 +269,14 @@ Boolean Groupsock::output(UsageEnvironment& env, u_int8_t ttlToSend,
     statsGroupOutgoing.countPacket(bufferSize);
     
     // Then, forward to our members:
-    int numMembers =
-      outputToAllMembersExcept(interfaceNotToFwdBackTo,
-			       ttlToSend, buffer, bufferSize,
-			       ourSourceAddressForMulticast(env));
-    if (numMembers < 0) break;
+    int numMembers = 0;
+    if (!members().IsEmpty()) {
+      numMembers =
+	outputToAllMembersExcept(interfaceNotToFwdBackTo,
+				 ttlToSend, buffer, bufferSize,
+				 ourSourceAddressForMulticast(env));
+      if (numMembers < 0) break;
+    }
     
     if (DebugLevel >= 3) {
       env << *this << ": wrote " << bufferSize << " bytes, ttl "
