@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // AMR Audio RTP Sources (RFC 3267)
 // Implementation
 
@@ -43,6 +43,7 @@ public:
   unsigned TOCSize() const { return fTOCSize; } // total # of frames in the last pkt
   unsigned char* TOC() const { return fTOC; } // FT+Q value for each TOC entry
   unsigned& frameIndex() { return fFrameIndex; } // index of frame-block within pkt
+  Boolean& isSynchronized() { return fIsSynchronized; }
 
 private:
   RawAMRRTPSource(UsageEnvironment& env, Groupsock* RTPgs,
@@ -55,18 +56,19 @@ private:
 
 private:
   // redefined virtual functions:
+  virtual Boolean hasBeenSynchronizedUsingRTCP();
+
   virtual Boolean processSpecialHeader(BufferedPacket* packet,
                                        unsigned& resultSpecialHeaderSize);
   virtual char const* MIMEtype() const; 
 
-  virtual Boolean hasBeenSynchronizedUsingRTCP();
-
 private:
   Boolean fIsWideband, fIsOctetAligned, fIsInterleaved, fCRCsArePresent;
   unsigned char fILL, fILP;
   unsigned fTOCSize;
   unsigned char* fTOC;
   unsigned fFrameIndex, fNumSuccessiveSyncedPackets;
+  Boolean fIsSynchronized;
 };
 
 class AMRDeinterleaver: public AMRAudioSource {
@@ -96,9 +98,10 @@ private:
   virtual void doStopGettingFrames();
 
 private:
-  FramedSource* fInputSource;
+  RawAMRRTPSource* fInputSource;
   class AMRDeinterleavingBuffer* fDeinterleavingBuffer;
   Boolean fNeedAFrame;
+  
 };
 
 
@@ -214,7 +217,7 @@ RawAMRRTPSource
   fIsWideband(isWideband), fIsOctetAligned(isOctetAligned),
   fIsInterleaved(isInterleaved), fCRCsArePresent(CRCsArePresent),
   fILL(0), fILP(0), fTOCSize(0), fTOC(NULL), fFrameIndex(0),
-  fNumSuccessiveSyncedPackets(0) {
+    fNumSuccessiveSyncedPackets(0), fIsSynchronized(false) {
 }
 
 RawAMRRTPSource::~RawAMRRTPSource() {
@@ -312,19 +315,11 @@ Boolean RawAMRRTPSource
 } 
 
 char const* RawAMRRTPSource::MIMEtype() const {
-  return fIsWideband ? "audio/AMR-WB" : "audio/AMR-WB";
+  return fIsWideband ? "audio/AMR-WB" : "audio/AMR";
 }
 
 Boolean RawAMRRTPSource::hasBeenSynchronizedUsingRTCP() {
-  // Don't report ourselves as being synchronized until we've received
-  // at least a complete interleave cycle of synchronized packets.
-  // This ensures that the receiver is currently getting a frame from
-  // a packet that was synchronized.
-  if (fNumSuccessiveSyncedPackets > (unsigned)(fILL+1)) {
-    fNumSuccessiveSyncedPackets = fILL + 2; // prevents overflow
-    return True;
-  }
-  return False;
+  return fIsSynchronized;
 }
 
 
@@ -401,7 +396,8 @@ public:
   Boolean retrieveFrame(unsigned char* to, unsigned maxSize,
 			unsigned& resultFrameSize, unsigned& resultNumTruncatedBytes,
 			u_int8_t& resultFrameHeader,
-			struct timeval& resultPresentationTime);
+			struct timeval& resultPresentationTime, 
+			Boolean& resultIsSynchronized);
 
   unsigned char* inputBuffer() { return fInputBuffer; }
   unsigned inputBufferSize() const { return AMR_MAX_FRAME_SIZE; }
@@ -418,6 +414,7 @@ private:
     unsigned char* frameData;
     u_int8_t frameHeader;
     struct timeval presentationTime;
+    Boolean fIsSynchronized;
   };
 
   unsigned fNumChannels, fMaxInterleaveGroupSize;
@@ -439,8 +436,7 @@ AMRDeinterleaver* AMRDeinterleaver
 ::createNew(UsageEnvironment& env,
 	    Boolean isWideband, unsigned numChannels, unsigned maxInterleaveGroupSize,
 	    RawAMRRTPSource* inputSource) {
-  return new AMRDeinterleaver(env, isWideband, numChannels, maxInterleaveGroupSize,
-			      inputSource);
+  return new AMRDeinterleaver(env, isWideband, numChannels, maxInterleaveGroupSize, inputSource);
 }
 
 AMRDeinterleaver::AMRDeinterleaver(UsageEnvironment& env,
@@ -464,7 +460,9 @@ void AMRDeinterleaver::doGetNextFrame() {
   // First, try getting a frame from the deinterleaving buffer:
   if (fDeinterleavingBuffer->retrieveFrame(fTo, fMaxSize,
 					   fFrameSize, fNumTruncatedBytes,
-					   fLastFrameHeader, fPresentationTime)) {
+					   fLastFrameHeader, fPresentationTime, 
+					   fInputSource->isSynchronized())) {
+
     // Success!
     fNeedAFrame = False;
 
@@ -597,6 +595,7 @@ void AMRDeinterleavingBuffer
   inBin.frameSize = frameSize;
   inBin.frameHeader = frameHeader;
   inBin.presentationTime = presentationTime;
+  inBin.fIsSynchronized = ((RTPSource*)source)->hasBeenSynchronizedUsingRTCP();
 
   if (curBuffer == NULL) curBuffer = createNewBuffer();
   fInputBuffer = curBuffer;
@@ -610,13 +609,16 @@ Boolean AMRDeinterleavingBuffer
 ::retrieveFrame(unsigned char* to, unsigned maxSize,
 		unsigned& resultFrameSize, unsigned& resultNumTruncatedBytes,
 		u_int8_t& resultFrameHeader,
-		struct timeval& resultPresentationTime) {
+		struct timeval& resultPresentationTime,
+		Boolean& resultIsSynchronized) {
+
   if (fNextOutgoingBin >= fOutgoingBinMax) return False; // none left
 
   FrameDescriptor& outBin = fFrames[fIncomingBankId^1][fNextOutgoingBin];
   unsigned char* fromPtr = outBin.frameData;
   unsigned char fromSize = outBin.frameSize;
   outBin.frameSize = 0; // for the next time this bin is used
+  resultIsSynchronized = outBin.fIsSynchronized;
 
   // Check whether this frame is missing; if so, return a FT_NO_DATA frame:
   if (fromSize == 0) {
