@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A simplified version of "MPEG4VideoStreamFramer" that takes only complete,
 // discrete frames (rather than an arbitrary byte stream) as input.
 // This avoids the parsing and data copying overhead of the full
@@ -83,7 +83,7 @@ void MPEG4VideoStreamDiscreteFramer
 	  break; // The configuration information ends here
 	}
       }
-      fNumConfigBytes = i-3;
+      fNumConfigBytes = i < frameSize ? i-3 : frameSize;
       delete[] fConfigBytes; fConfigBytes = new unsigned char[fNumConfigBytes];
       for (unsigned j = 0; j < fNumConfigBytes; ++j) fConfigBytes[j] = fTo[j];
 
