@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // RTCP
 // Implementation
 
@@ -349,7 +349,7 @@ void RTCPInstance::incomingReportHandler1() {
       // NOTE: Denial-of-service attacks are possible here.
       // Users of this software may wish to add their own,
       // application-specific mechanism for 'authenticating' the
-      // validity of this packet before relecting it.
+      // validity of this packet before reflecting it.
       fRTCPInterface.sendPacket(pkt, packetSize);
       fHaveJustSentPacket = True;
       fLastPacketSentSize = packetSize;
@@ -583,8 +583,10 @@ void RTCPInstance::onReceive(int typeOfPacket, int totPacketSize,
 }
 
 void RTCPInstance::sendReport() {
-  /// Note: Don't send a SR until at least one RTP packet has been sent. (David Bertrand, 2006.07.18)
-  if (fSink != NULL && !fSink->haveComputedFirstTimestamp()) return;
+  // Hack: Don't send a SR during those (brief) times when the timestamp of the
+  // next outgoing RTP packet has been preset, to ensure that that timestamp gets
+  // used for that outgoing packet. (David Bertrand, 2006.07.18)
+  if (fSink != NULL && fSink->nextTimestampHasBeenPreset()) return;
 
 #ifdef DEBUG
   fprintf(stderr, "sending REPORT\n");
