@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
-// Copyright (c) 1996-2006, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2007, Live Networks, Inc.  All rights reserved
 // A RTSP client test program that opens a RTSP URL argument,
 // and extracts the data from each incoming RTP stream.
 
@@ -60,9 +60,18 @@ Boolean clientSetupSubsession(Medium* client, MediaSubsession* subsession,
 
 Boolean clientStartPlayingSession(Medium* client,
 				  MediaSession* session) {
+  extern double initialSeekTime, duration, scale;
+  double endTime = initialSeekTime;
+  if (scale > 0) {
+    endTime = initialSeekTime + duration;
+  } else if (scale < 0) {
+    endTime = initialSeekTime - duration;
+    if (endTime < 0) endTime = 0.0f;
+  }
+
   if (client == NULL || session == NULL) return False;
   RTSPClient* rtspClient = (RTSPClient*)client;
-  return rtspClient->playMediaSession(*session);
+  return rtspClient->playMediaSession(*session, (float)initialSeekTime, (float)endTime, (float)scale);
 }
 
 Boolean clientTearDownSession(Medium* client,
